/*******************************************************************************/
/*                                                                             */
/*   common.c :  JPL DEt@CpɂVvZ̊{vZ֐                */
/*      Ver. 3.0   2017/08/17         By    takesako@mrj.biglobe.ne.jp         */
/*                                           (c)1999 Shinobu Takesako          */
/*******************************************************************************/
/*   1999/06/29 avZǉɔC                                         */
/*   1999/07/02 PʒuvZǉɔǉ                                     */
/*              PrecessioñoOC                                           */
/*   2016/10/10 ֐̒ǉ													   */
/*              DateToYMDH(), HourToHMS(), Observer(), fmod_deg(), Point()     */
/*   2017/08/17 vZԒZkׁ̈Aꕔradianx[XɕύXB           */
/*              폜F Compact(),Table_Value(),C_1900(),C_1800(),Obl()         */
/*              ǉF OblNu_2000(), Aberration(),  Star_posi()                */
/*              SiderealTime(), Star_posi_ALG(),Star_posi_ALM()                */
/*              CF LST(),LST2(),Obl0_2000(),Obl_2000(),LamToRa(),          */
/*                     RaToLam(),Az_El() Nutation֌WC                */
/*   2019/06/21 Precession()̌ŗL^萔sky typeɕύX                     */
/*******************************************************************************/

#include <stdio.h>
#include <math.h>
#include "astro.h"

/*******************************************************************************/
/*                      EX̌vZ        (revised 2015/03/16)           */
/*                    @(jd<0 ɑΉ)                                          */
/*               gpԂnW(LST)ȂF LST=1                         */
/*               gpԂnώ(LMT)ȂF LST=0                         */
/*******************************************************************************/

double JD(double year,double month,double day,double hour,int LST,double longitude)
{
	double s,juday,a,flag;

	flag =1;
	if (longitude <0){ 
	longitude = -longitude;
	flag=-1;
	}

	if ( month < 3 ){
	year = year -1;
	month = month +12;
	};
	s = floor( 365.25 * year) + floor( 30.59 * (month-2.0));
	if (LST==1)
	{juday = s + day + hour/24.0 -  flag*floor(longitude /15.0+0.5)/24.0	+ 1721086.5;}
	else
	{juday = s + day + hour/24.0 -  flag*longitude /15.0/24.0	+ 1721086.5;}

	if ( juday > 2299161 ) { 
	a = floor ( year / 400 ) - floor( year / 100)+2;
	juday = juday + a;
	};

	return(juday);
}

/*******************************************************************************/
/*                      ̌vZ        (revised 2015/03/16)                 */
/*                      (jd<0 ɑΉ)                                          */
/*               gpԂnW(LST)ȂF LST=1                         */
/*               gpԂnώ(LMT)ȂF LST=0                         */
/*******************************************************************************/

void Calendar( double juday, double *year,double *month,double *day,double *hour,int LST,double longitude)
{
	double flag,Z,Alfa,A,B,C,D,E,F,Day0;
	
	flag = 1;

	if (longitude<0 ){
	longitude = - longitude;
	flag=-1.0;
	}
	
	if (LST==1) juday =  juday +  flag*(double)(int)(longitude /15.0+0.5)/24.0;
	else    juday =  juday +  flag*longitude /15.0/24.0	;

	juday = juday + 0.5;
	Z = floor(juday); 
	F = juday - Z;

	if (Z < 2299161.0) A = Z;
	else { 
	Alfa = (double)(int) ((Z - 1867216.25)/36524.25);
	A = Z + 1 + Alfa - (double)(int) (Alfa/4.0);}

	B = A + 1524;
	C = floor ((B-122.1)/365.25);
	D = floor (365.25 * C);
	E = (double)(int) ((B-D)/30.6001);

	Day0 = B - D - floor(30.60001*E) + F;
	*day = (double)(int)(Day0);
	*hour= F*24.0;
	if(E<14.0) *month = E-1.0;
	if(E==14.0 || E==15.0)	 *month = E- 13.0;
	if(*month>2.0) *year=C-4716;
	if(*month==1.0 || *month==2.0) *year = C - 4715;
}

/*******************************************************************************/
/*  nP(Local SiderealTime)̌vZ1 (Astronomical Almanac 2014 B8ɂ) */
/*			(2017/08/10	C)			 ( gpP:radian )                   */
/*******************************************************************************/

double LST( double jd, double longitude)
{
	double lst;

	lst=SiderealTime(jd)+longitude*pai;
	
	return lst;
}

/*******************************************************************************/
/*     P(SiderealTime)̌vZ1    (Astronomical Almanac 2014 B8ɂ)     */
/*			(2017/08/10	ǉ)			 ( gpP:radian )                   */
/*******************************************************************************/

double SiderealTime(double jd)
{
double Theta,Theta0,dF,dE,Eq,Obl,T,st;
static double data[6]={0.014506, 4612.156534,1.3915817,  - 0.00000044, - 0.000029956, -3.68e-8};
int i;

	T=T_2000 (jd);
	Theta  = 360.0 * (0.7790572732640 + 1.00273781191135448*(T*36525.0));
	Theta0=0.0;
	for(i=1;i<=5;i++){
	Theta0=(Theta0+data[6-i])*T;
	}
	Theta0=(Theta0+data[0])/3600.0;
	st=fmod_deg(Theta + Theta0)*pai;

	Obl=OblNu_2000( jd, &dF, &dE);
	Eq=dF*cos(Obl);

	st=st + Eq;
	return st;
}

/*******************************************************************************/
/*                      QOOON̐ǏvZ                           */
/*******************************************************************************/

double T_2000 (double jd)
{
	double JD0=2451545.0 , t ;
	t=(jd-JD0)/(100.0*365.25);
	return t ;
}

/*******************************************************************************/
/*       XΊp̌vZ  }3000Nŗ͖̍1"                            */
/*       }3000N܂Ł@(Base on Astromical Almanac 2014 B52)                   */ 
/*                                                                             */
/*       }3000Nȍ~  (Base on Astromical Algorithms 2nd p.147)               */
/*       Top 2414'07" at -7530, bottom 2236'41"  at +12030                 */
/*       (̕ۏ؂Ă͈͂́}1N)                                       */
/*       덷F 0.01"}1000N@b}1N ( gpP:radian )                 */
/*******************************************************************************/

double Obl0_2000(double jd) // without Nutation  2017/08/10 mod
{
	double t,obl;
	int i;

	static double data0[11]={23.4392794444, -46.836769, -0.0001831, 0.00200340, -0.576e-6, -4.34e-8, 0, 0, 0, 0, 0};
	static double data1[11]={84381.448, -4680.93, -1.55, 1999.25, -51.38, -249.67, -39.05, 7.12, 27.87, 5.79, 2.45};

	t=T_2000(jd);

	if(fabs(t)<=30.0){
	obl=0.0;
	for(i=1;i<=10-5;i++){
	obl=(obl+data0[11-i-5])*t;
	}
	obl=(data0[0]+obl/3600.0)*pai;
	}
	else
	{
	t=t/100;
	obl=0.0;
	for(i=1;i<=10;i++){
	obl=(obl+data1[11-i])*t;
	}
	obl=(obl+data1[0])/3600.0*pai;
	}

	return obl;
}

double Obl_2000(double jd) // with Nutation
{
	double dF,dE,obl;

	Nutation( jd, &dF, &dE); // 2017/08/10 mod
	obl = Obl0_2000(jd) +dE; // 2017/08/10 mod
     
	return obl;
}

double OblNu_2000(double jd,double *dF,double *dE) // with Nutation
{
	double dF0,dE0,obl;

	Nutation( jd, &dF0, &dE0); // 2017/08/10 mod
	obl = Obl0_2000(jd) +dE0;  // 2017/08/10 mod
     
	*dF=dF0;
	*dE=dE0;
	return obl;
}

/*******************************************************************************/
/*                      o܂ԌoԈ܂ւ̕ϊ      ( gpP:radian )  */
/*						2017/08/10 radian֕ύX                                */
/*******************************************************************************/

void LamToRa(double jd,double Lam,double Bet,double *RA,double *DEC)
{
	double Obl,sinObl,cosObl,sinLam,cosLam,sinBet,cosBet;
	
	Obl = Obl_2000(jd);
	sinObl=sin(Obl);
	cosObl=cos(Obl);
	sinLam=sin(Lam);
	cosLam=cos(Lam);
	sinBet=sin(Bet);
	cosBet=cos(Bet);
	
	*RA  = fmod_rad(atan2((cosObl*cosBet*sinLam-sinObl*sinBet),cosBet*cosLam));
	*DEC = asin (cosObl*sinBet+sinObl*cosBet*sinLam);
}

/*******************************************************************************/
/*                      ԌoԈ܂物o܂ւ̕ϊ       ( gpP:radian ) */
/*						2017/08/10 radian֕ύX                                */
/*******************************************************************************/

void RaToLam(double jd,double RA,double DEC,double *Lam,double *Bet)
{
	double Obl,sinObl,cosObl,sinRA,cosRA,sinDEC,cosDEC;
	
	Obl = Obl_2000(jd);
	sinObl=sin(Obl);
	cosObl=cos(Obl);
	sinRA=sin(RA);
	cosRA=cos(RA);
	sinDEC=sin(DEC);
	cosDEC=cos(DEC);

	*Lam  = fmod_rad(atan2((sinObl*sinDEC+cosObl*cosDEC*sinRA),cosDEC*cosRA));
	if (*Lam <0) *Lam = *Lam +360.0;
	*Bet = asin (cosObl*sinDEC-sinObl*cosDEC*sinRA);
}

/*******************************************************************************/
/*                      ԌoԈ܂ʍx̌vZ         ( gpP:radian ) */
/*						2017/08/10 radian֕ύX                                */
/*******************************************************************************/
 
void Az_El(double jd,double Ra, double Dec, double latitude,double longitude,  double *Az, double *El )
{
	double Lst,Lat,Ha,a,b,sinLst,cosLst,sinLat,cosLat,sinHa,cosHa,sinDec,cosDec;

    Lst = LST(jd,longitude);
	Lat = (latitude)*pai;
	Ha  = (Lst - Ra);// 2017/08/10 C
	
	sinLst=sin(Lst);
	cosLst=cos(Lst);
	sinLat=sin(Lat);
	cosLat=cos(Lat);
	sinHa=sin(Ha);
	cosHa=cos(Ha);
	sinDec=sin(Dec);
	cosDec=cos(Dec);

	a   = sinHa;
	b   = cosHa * sinLat - sinDec/cosDec*cosLat;
	*Az = atan2 (a,b);
	*El =asin(sinLat * sinDec + cosLat* cosDec*cosHa);
}

/*******************************************************************************/
/*                      tϐN̕                               */
/*******************************************************************************/

void DateToYMD(double Date,double *year,double *month,double *day)

{
	double flag,Year,Month;
	
	flag=0;
	
	if (Date<0) {
	Date=-Date;
	    flag = 1;
	    };
	Year  = floor(Date);
	Month = floor((Date-Year)*100);
	*day  = (Date*10000-Year*10000-Month*100);
	*year = Year;
	*month= Month;
	if(flag==1) *year = -*year;
}

/*******************************************************************************/
/*                      tϐN̕                             */
/*					     (2016/09/08 ǉ)									   */
/*******************************************************************************/

void DateToYMDH(double Date,double *year,double *month,double *day,double *hour)
{
	double flag,Year,Month,min,sec;
	
	flag=0;
	
	if (Date<0) {
		Date=-Date;
	    flag = 1;
	    };
	Year  = floor(Date);
	Month = floor((Date-Year)*100);
	*day  = (Date*10000-Year*10000-Month*100);
	*hour = (*day-floor(*day+0.001))*100.0;
	min  =(*hour-floor(*hour))*100.0;
	sec  =floor((min-floor(min))*100.0+0.5/60.0);
	if(sec>=65.0)sec=0.0;
	min  =floor(min+0.5/60.0);
	if(min>=65.0)min=0.0;

	*hour=floor(*hour+0.5/60.0);
	*hour=*hour+min/60.0+sec/3600.0;
	*day  = floor(*day);
	*year = Year;
	*month= Month;
	if(flag==1) *year = -*year;
}

/*******************************************************************************/
/*                      ԕb\玞Ԃւ̕ϊ                           */
/*******************************************************************************/
double HMSToHour(double HMS)

{
	int flag=0;
	double H,Hour,Min,Sec;
	
	if (HMS<0){
	flag=1;
	HMS=-HMS;
	};
	Hour  = floor(HMS);
	Min   = floor((HMS - Hour)*100);
	Sec   = (HMS*10000-Hour*10000-Min*100);
	H     = Hour + Min/60.0 +  Sec/3600.0;
    if(flag==1)H=-H;
	return H;
}

/*******************************************************************************/
/*                      ԕb\pxւ̕ϊ  ( 1999/07/01 ǉ )      */
/*******************************************************************************/
double HMSToDeg(double HMS)

{
	double H,Deg;
	
	H=HMSToHour( HMS);
	
	Deg = H*15.0;
	return Deg;
}

/*******************************************************************************/
/*                      xb\xւ̕ϊ                               */
/*******************************************************************************/

double DMSToDeg(double DMS)

{
	int flag=0;
	double D,Deg,Min,Sec;
	
	if (DMS<0){
	flag=1;
	DMS=-DMS;
	};
	Deg   = floor(DMS);
	Min   = floor((DMS - Deg)*100);
	if(Min>99.0){Min=Min-100;Deg=Deg+1;}
	Sec   = DMS*10000-Deg*10000-Min*100;
	if(Sec>99.0){Sec=Sec-100;Min=Min+1;}
	D     = Deg +  Min/60.0 +  Sec/3600.0;
	if(flag==1)D=-D;
    return D;
}

/*******************************************************************************/
/*                      x\玞b\ւ̕ϊ                           */
/*******************************************************************************/

double DegToHMS(double D)

{
	int    flag=0;
	double Hour,Min, Sec,H;
	
	if(D<0) {
	D=-D;
	flag=1;
	};

	D     = D/15.0;
	Hour  = floor(D);
	Min   = floor((D - Hour)*60);
	Sec   = D*3600-Hour*3600-Min*60;
	H     = Hour +  Min/100.0 +  Sec/10000.0;
	if(flag==1)H=-H;

    return H;
}
/*******************************************************************************/
/*                      x\玞b\ւ̕ϊ                           */
/*******************************************************************************/

double HourToHMS(double D)

{
	int    flag=0;
	double Hour,Min, Sec,H;
	
	if(D<0) {
	D=-D;
	flag=1;
	};

	Hour  = floor(D+0.000001);
	Min   = floor((D - Hour)*60);
	Sec   = D*3600-Hour*3600-Min*60;
	if(Sec>59.999){Sec=Sec-60.0;Min=Min+1.0;}
	H     = Hour +  Min/100.0 +  Sec/10000.0;
	if(flag==1)H=-H;

    return H;
}
/*******************************************************************************/
/*                      x\xb\ւ̕ϊ                           */
/*******************************************************************************/

double DegToDMS(double D)

{
	int    flag=0;
	double Deg,Min,Sec,H;
	
	if(D<0) {
	D=-D;
	flag=1;
	};
	Deg   = floor(D);
	Min   = floor((D - Deg)*60.0);
	Sec   = (D*3600-Deg*3600-Min*60);
	H     = Deg +  Min/100.0 + Sec/10000.0;
	if(flag==1)H=-H;
    return H;
}

/*******************************************************************************/
/*                      _ʂł̎ľܓ                               */
/*******************************************************************************/

double Digit2( double x)
{
	int flag=0;

	if(x<0){
	x=-x;
	flag=1;
	}
	x=(double)((int)(x*100.0+0.5));
	x=x/100.0;
	if(flag==1)x=-x;
	return x;
}

/*******************************************************************************/
/*                      _Oʂł̎ľܓ                               */
/*******************************************************************************/

double Digit3( double x)
{
	int flag=0;

	if(x<0){
	x=-x;
	flag=1;
	}
	x=(double)((int)(x*1000.0+0.5));
	x=x/1000.0;
	if(flag==1)x=-x;
	return x;
}

/********************************************************************************/
/*                     px360ȓւ̏k   (2016/09/031ǉ)              */
/*										                                        */
/********************************************************************************/

double fmod_deg(double deg)
{
	deg=fmod(deg,360.0);
	if(deg<0)deg=deg+360.0;
	if(deg==360.0)deg=0.0;
	return deg;
}

/********************************************************************************/
/*                     px2Έȓւ̏k   (2017/08/11ǉ)                 */
/*										                                        */
/********************************************************************************/

double fmod_rad(double deg)
{
	deg=fmod(deg,pi2);
	if(deg<0)deg=deg+pi2;
	if(deg==pi2)deg=0.0;
	return deg;
}

/*******************************************************************************/
/*                     ϑn␳   (2016/08/31ǉ)     ( gpP:degree )   */
/*              (Astronomical algorithms 2nd p.82ɂ)                       */
/*******************************************************************************/
void Observer(double latitude, double longitude, double H,Tlocation *Obs)
{
	double a,b,c,f,u;

	Obs->fai=latitude;
	Obs->lam=longitude;

	a=6378.14;
	f=1.0/298.257;
	b=a*(1-f);
	c=b/a;

	u=atan(c*tan(Obs->fai*pai));
	Obs->psinfd=c*sin(u) + H/6378140.0*sin(Obs->fai*pai);
	Obs->pcosfd=  cos(u) + H/6378140.0*cos(Obs->fai*pai); 
}

/*******************************************************************************/
/*                     ΍̌vZ iN̕ωɑΉj   ( gpP:degree ) */
/*              (Astronomical algorithms 2nd .134ɂ)                       */
/* 2019/06/21 sky typeɕύX                                                   */
/*******************************************************************************/

void Precession (double jd, double Year0, double RA0,double DEC0,double V1,double V2,double *RA,double *DEC)
{
	double jd0,t,T, Zeta,Zeta1,Zeta2,Zeta3,Theta,Theta1,Theta2,Theta3,Zed,Zed1,Zed2,Zed3;
	double CosTheta,SinTheta ,CosDec0,SinDec0,CosRaZeta ,SinRaZeta;

	jd0=JD( Year0,1,1,12,0,0);/*1999/07/02*/

	T=(jd0-2451545.0)/36525.0;/*1999/07/02*/
	t=(jd-jd0)/36525.0;

	Zeta1  = 2306.2181 + 1.39656*T - 0.000139*T*T;
	Zeta2  = 0.30188 - 0.000344*T;
	Zeta3  = 0.017998;
	Theta1 = 2004.3109 - 0.85330*T - 0.000217*T*T;
	Theta2 = - ( 0.42665 + 0.000217*T);
	Theta3 = - 0.041833;
	Zed1   = 2306.2181 + 1.39656*T - 0.000139*T*T;
	Zed2   = 1.09468 + 0.000066*T;
	Zed3   = 0.018203;

	Zeta  = (  Zeta1*t +  Zeta2*t*t + Zeta3*t*t*t )/3600.0;
	Zed   = (   Zed1*t +   Zed2*t*t +  Zed3*t*t*t )/3600.0;
	Theta = ( Theta1*t + Theta2*t*t +Theta3*t*t*t )/3600.0;

	Zeta = Zeta*pai;
	Zed  = Zed*pai;
	Theta= Theta*pai;
//	RA0  = (RA0 + V1*t     *100/3600.0/cos(DEC0*pai))*pai;// BSC type
	RA0  = (RA0 + V1*t*15.0*100/3600.0)*pai;              // sky2000 type 2019/06/21

	DEC0 = (DEC0+ V2*t*100/3600.0)*pai;

	CosTheta  = cos (Theta);
	SinTheta  = sin (Theta);
	CosDec0   = cos (DEC0);
	SinDec0   = sin (DEC0);
	CosRaZeta = cos(RA0+Zeta);
	SinRaZeta = sin(RA0+Zeta);

	*DEC   = asin (CosTheta*SinDec0 + SinTheta*CosDec0*CosRaZeta)/pai;
	*RA =atan2(CosDec0*SinRaZeta ,CosTheta*CosDec0*CosRaZeta-SinTheta*SinDec0)/pai;
	*RA =*RA + Zed/pai;
	if (*RA<0) *RA += 360.0;
}

/********************************************************************************/
/*                     ΍̌vZ iNF2000Nj   ( gpP:degree )      */
/*                     (J2000N̏ɂ)                                    */
/*              (Astronomical algorithms 2nd p.134ɂ)                       */
/********************************************************************************/

void Point(double jd, double RA0, double DEC0, double V1, double V2, double *RA, double *DEC,int type)
{
	double T,ZETA0, ZED, THETA;
	double CosT, SinT, CosD, SinD, CosR, SinR,k=1.0,dRa,dDec;

	T=(jd-2451545.0)/36525.0;
//printf(" T=%15.8f \n",T);

	// BSC type
	if(type==0){
	dRa  = (V1*T*100/3600.0/(cos(DEC0*pai)));
	dDec = (V2*T*100/3600.0);
	}
	// sky2000 type
	else
	{
	dRa  = (V1*15.0*T*100/3600.0);
	dDec = (V2     *T*100/3600.0);
	}

printf(" dRa=%15.8f dDec=%15.8f ŗL^\n",(dRa)*3600,(dDec)*3600);

	RA0  = (RA0 + dRa )*pai;
	DEC0 = (DEC0+ dDec)*pai;

	ZETA0 = (2306.2181*T + 0.30188*T*T + 0.017998*T*T*T )/3600.0 * pai;
	ZED   = (2306.2181*T + 1.09468*T*T + 0.018203*T*T*T )/3600.0 * pai;
	THETA = (2004.3109*T - 0.42665*T*T - 0.041833*T*T*T )/3600.0 * pai;

	CosT = cos (THETA);
	SinT = sin (THETA);
	CosD = cos (DEC0);
	SinD = sin (DEC0);
	CosR = cos(RA0+ZETA0);
	SinR = sin(RA0+ZETA0);

	*DEC = asin (CosT*SinD+SinT*CosD*CosR);
	*RA  = atan2(CosD*SinR,CosT*CosD*CosR-SinT*SinD);
	*RA = *RA + ZED;

	*RA  = fmod_deg( *RA  /pai);
	*DEC = *DEC /pai;

printf(" dRa=%15.8f dDec=%15.8f ΍\n",(*RA-RA0/pai)*3600,(*DEC-DEC0/pai)*3600);

}

/********************************************************************************/
/*  siAberration of lightjɂԌoԈ܂̕ω̌vZ  (2017/08/10ǉ) */
/*         (Astronomical algorithms 2nd p.151-153ɂ)    ( gpP:radian ) */
/********************************************************************************/

void Aberration(double jd, double Obl, double RA, double DEC,double *dRA,double *dDEC)
{
	double	t, L0, M, C, ec, ec2, lams, PI, coslams, sinlams,k;
	double	sinRA, cosRA, sinDEC, cosDEC, sinObl, cosObl;

	t=(jd-2451545.0)/36525.0;

	//zǑvZ	
	L0 = fmod((280.46646 + (36000.76983 + 0.0003032*t)*t),360.0)*pai;
	M  = fmod((357.52911 + (35999.05029 -  0.0001537*t)*t),360.0) * pai;
	ec = 0.016708634 - (0.000042037 + 0.0000001267*t)*t;
	ec2= ec * ec;
	C  = ((1.914602-0.004817*t-0.000014*t*t)*sin(M)+(0.019993-0.000101*t)*sin(2.0*M)+0.000289*sin(3.0*M))*pai;
	lams = L0 + C;

    // E-Term̌vZ(ec̍ځAFK4ł͕sv)
//	PI = (102.93735 + (0.71954 + 0.00046*t)*t) * pai;// v1.0 version
	PI = (102.93735 + (1.71946 + 0.00046*t)*t) * pai;// based on Astronomical algorithms v2.0 corrected from 0.71954 to 1.71946 
	coslams = cos(lams) - ec * cos (PI);
	sinlams = sin(lams) - ec * sin (PI);

	k=20.49552/3600.0*pai;// constant of Aberration

	sinRA=sin(RA);
	cosRA=cos(RA);
	sinDEC=sin(DEC);
	cosDEC=cos(DEC);
	sinObl=sin(Obl);
	cosObl=cos(Obl);

	*dRA=-k*(cosRA*coslams*cosObl+sinRA*sinlams)/cosDEC;
	*dDEC=-k*(coslams * (sinObl * cosDEC - sinRA * sinDEC * cosObl) + cosRA * sinDEC * sinlams);
}
/********************************************************************************/
/*             mȐ̈ʒǔvZ iNF2000Nj   ( gpP:degree )    */
/*						(2017/08/17ǉ)	                                    */
/*              (Astronomical algorithms 2nd p.149-153ɂ)                   */
/********************************************************************************/

void Star_posi_ALG(double jd, double RA0, double DEC0, double V1, double V2, double *RA, double *DEC,int type)
{
	double Ra,Dec,Obl,dRA1,dDEC1,dRA2,dDEC2,dF,dE;

	// ΍̌vZ(2000N̐\̒l)
	Point( jd, RA0, DEC0, V1, V2, &Ra, &Dec,type);
	Ra =Ra *pai;
	Dec=Dec*pai;

	// ͓̌vZ(IAU2000B)
	Obl=OblNu_2000(jd,&dF,&dE) ;// with Nutation

	// ͓ɂω̌vZ
	Nutation_delta( jd, Obl, dF, dE, Ra, Dec, &dRA1, &dDEC1);
	printf(" dRa=%15.8f dDec=%15.8f ͓(Nutation)\n",(dRA1/pai)*3600,(dDEC1/pai)*3600);
	Ra=Ra+(dRA1);
	Dec=Dec+(dDEC1);

	// ɂω̌vZ
	Aberration( jd, Obl, Ra, Dec, &dRA2,&dDEC2);
	printf(" dRa=%15.8f dDec=%15.8f \n",(dRA2/pai)*3600,(dDEC2/pai)*3600);

	*RA=(Ra+(dRA2))/pai;
	*DEC=(Dec+(dDEC2))/pai;
	printf(" dRa=%15.8f dDec=%15.8f v\n",(*RA-RA0)*3600,(*DEC-DEC0)*3600);

}


/********************************************************************************/
/*             mȐ̈ʒǔvZ iNF2000Nj   ( gpP:degree )    */
/*				 (2017/08/17ǉ)							                    */
/*              (Astronomical Almanac 2014 B72-B74ɂ)                       */
/********************************************************************************/

void Star_posi_ALM(FILE *fp_Bin, int DE, double jd, double RA0, double DEC0, double ura, double udec, double Pi, double v, double *Ra, double *Dec)

{
	double T,Obl;
	double RA, DEC,dF,dE, vpi,sinra,cosra,sindec,cosdec,RA1,DEC1,RA2,DEC2;
	double Es,Ps,pes,muc,Vs,bet,betm,pp4Vs,pvc;
	int Planet;

	Matrix_1 q,q1,q2,p,p1,p2,p3,pp2,pp3,pp4,pp5,pp6,pp7,mu,Eb,Ebv,Sb,P,E,e,V;
	Matrix_3 Ppn;
	Position Eph;

//	Step 1.
	T=T_2000(jd);

//	Step 2. ŗL^̌vZ
	Planet=3;
	Ephemerides( fp_Bin,DE,jd, Planet, &Eph);
	Eb.max[0] =Eph.Ebx ;Eb.max[1] =Eph.Eby ;Eb.max[2]=Eph.Ebz;
	Ebv.max[0]=Eph.Ebvx;Ebv.max[1]=Eph.Ebvy;Ebv.max[2]=Eph.Ebvz;
	Sb.max[0] =Eph.Sbx ;Sb.max[1] =Eph.Sby ;Sb.max[2]=Eph.Sbz;

	RA0=RA0*pai;
	DEC0=DEC0*pai;
	ura =ura /(1000*3600.0)*100*pai;//100year
	udec=udec/(1000*3600.0)*100*pai;//100year

	Pi=Pi/3600.0*pai;
	v=  v*(3600.0*24.0*36525/149597870.700);
	vpi=v*Pi;
	CordToMatrix_1( RA0 , DEC0, &q);

	sinra=sin(RA0);
	cosra=cos(RA0);
	sindec=sin(DEC0);
	cosdec=cos(DEC0);
	mu.max[0]=-ura*sinra-udec*sindec*cosra+vpi*cosdec*cosra;
	mu.max[1]= ura*cosra-udec*sindec*sinra+vpi*cosdec*sinra;
	mu.max[2]=           udec*cosdec      +vpi*sindec      ;

	Multi_Const_Matrix_1( mu, T, &q1);
	Multi_Const_Matrix_1( Eb, Pi, &q2);

	Sum_Matrix_1( q , q1, &P);
	Sub_Matrix_1( P , q2, &P);
	Scalar ( P, &Ps);

	Sub_Matrix_1(Eb,Sb,&E);
	Scalar ( E, &Es);

	p.max[0]=P.max[0]/Ps; p.max[1]=P.max[1]/Ps, p.max[2]=P.max[2]/Ps;
	e.max[0]=E.max[0]/Es; e.max[1]=E.max[1]/Es, e.max[2]=E.max[2]/Es;

	MatrixToCord_Matrix_1( p, &RA1 , &DEC1);
	printf(" dRA =%15.8fdDec =%15.8f ŗL^\n",((RA1-RA0)/pai*3600),((DEC1-DEC0)/pai*3600));

//	Step 3.@̌vZ
	Scalar_Dot ( p,e, &pes);

	Multi_Const_Matrix_1( p, pes, &p1);
	Sub_Matrix_1(e,p1,&pp2);
	muc=2.0*9.87e-9*Es/(1+pes);// mu/(c*c)=9.87e-9
	Multi_Const_Matrix_1( pp2, muc, &pp3);
	Sum_Matrix_1(p,pp3,&pp4);

//	Step 4.
	Multi_Const_Matrix_1( Ebv, 0.005775518, &V);
	Scalar ( V, &Vs);
	bet=1/sqrt(1-Vs*Vs);
	betm=1/bet;

	Scalar_Dot ( pp4,V, &pp4Vs);
	pvc=1+pp4Vs/(1+betm);

	Multi_Const_Matrix_1( pp4, betm, &pp5);
	Multi_Const_Matrix_1( V, pvc, &pp6);
	Sum_Matrix_1(pp5,pp6,&pp7);
	Multi_Const_Matrix_1( pp7, 1/(1+pp4Vs), &p2);

	MatrixToCord_Matrix_1( p2, &RA2 , &DEC2);
	printf(" dRA =%15.8fdDec =%15.8f \n",((RA2-RA1)/pai*3600),((DEC2-DEC1)/pai*3600));

//	Step 5.@΍͓̌vZ
	Obl=OblNu_2000( jd, &dF, &dE);
	Nutation_Precession_Matrix( jd,&Ppn,Obl,dE,dF);
	Multi_Matrix_1(Ppn,p2,&p3);

//	Step 6.
	MatrixToCord_Matrix_1( p3, &RA , &DEC);
	printf(" dRA =%15.8fdDec =%15.8f ΍͓\n",((RA-RA2)/pai*3600),((DEC-DEC2)/pai*3600));
	printf(" dRA =%15.8fdDec =%15.8f v\n",((RA-RA0)/pai*3600),((DEC-DEC0)/pai*3600));

	*Ra=RA/pai;
	*Dec=DEC/pai;

}


/********************************************************************************/
/*                     z//f̏ov̌vZ  (2017/08/10ǉ)               */
/********************************************************************************/

void RiseSet(FILE *fp_Bin,int DE,double jd, int Planet,int LST, Tlocation Obs,double *rise ,double *set)
{
	double year,month,day,hour,longitude,eph[5],jds,delta,hs;
	int i;
	Position Eph;

	longitude=Obs.lam;

	Calendar( jd, &year, &month, &day, &hour, LST, longitude);
	if(Planet==11) hs=(35.8+16.0)/60.0;//0.8x
	else hs=(35.8+0.0)/60.0;//f̏ꍇ

	//oṽ|Cgo
	for(i=0;i<=4;i++){
	hour=(double)i*6;
	jds=JD(year, month, day, hour, LST,longitude);
	Ephemerides2( fp_Bin,DE,jds, Planet, &Eph,Obs);
	eph[i]=Eph.el/pai;;
	}

	//ǒvZ
	for(i=0;i<=3;i++){
	if(eph[i]<0.0 && eph[i+1]>0.0 ) break;
	}
	hour=(double)i*6.0+3.0;
	delta=1;
	jd=JD(year, month, day,hour, LST,longitude);// jd͐E(UT)
	while (fabs(delta)> 0.00001){
	Ephemerides2( fp_Bin,DE,jd, Planet, &Eph,Obs);//  V̂̈ʒuvZ
	delta=(Eph.el/pai+hs)/12.0/24.0;
	jd=jd-delta;
	}
	Calendar( jd, &year,&month,&day,&hour,LST, longitude);
	*rise=hour;

	//v̌vZ
	for(i=0;i<=3;i++){
	if(eph[i]>0.0 && eph[i+1]<0.0 ) break;
	}
	hour=(double)i*6.0+3.0;
	delta=1;
	jd=JD(year, month, day, hour, LST,longitude);// jd͐E(UT)
	while (fabs(delta)> 0.00001){
	Ephemerides2( fp_Bin,DE,jd, Planet, &Eph,Obs);//  V̂̈ʒuvZ
	delta=(Eph.el/pai+hs)/12.0/24.0;
	jd=jd+delta;
	}
	Calendar( jd, &year,&month,&day,&hour,LST, longitude);
	*set=hour;
}